@@ -16,15 +16,14 @@ define('IMP_VFS_LINK_ATTACH_PATH', '.horde/imp/attachments');
  * The IMP_Compose:: class contains functions related to generating
  * outgoing mail messages.
  *
- * $Horde: imp/lib/Compose.php,v 1.107 2004/12/12 06:36:52 slusarz Exp $
+ * $Horde: imp/lib/Compose.php,v 1.107.2.16 2005/07/03 03:42:42 selsky Exp $
  *
- * Copyright 2002-2004 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.107 $
  * @since   IMP 4.0
  * @package IMP
  */
@@ -33,30 +32,28 @@ class IMP_Compose {
     /**
      * The cached attachment data.
      *
-     * @var array $_cache
+     * @var array
      */
     var $_cache = array();
 
     /**
      * For findBody, the MIME ID of the "body" part.
      *
-     * @var string $_mimeid
+     * @var string
      */
     var $_mimeid = null;
 
     /**
      * The aggregate size of all attachments (in bytes).
      *
-     * @var integer $_size
+     * @var integer
      */
     var $_size = 0;
 
     /**
      * Constructor
      *
-     * @access public
-     *
-     * @param optional array $params  Parameters needed.
+     * @param array $params  Parameters needed.
      */
     function IMP_Compose($params = array())
     {
@@ -68,16 +65,14 @@ class IMP_Compose {
     /**
      * Send a message.
      *
-     * @access public
-     *
-     * @param string $email                 The e-mail list to send to.
-     * @param object IMP_Headers &$headers  The IMP_Headers object holding
-     *                                      this messages headers.
-     * @param mixed &$message               Either the message text (string)
-     *                                      or a MIME_Message object that
-     *                                      contains the text to send.
-     * @param string $charset               The charset that was used for the
-     *                                      headers.
+     * @param string       $email    The e-mail list to send to.
+     * @param IMP_Headers &$headers  The IMP_Headers object holding
+     *                               this messages headers.
+     * @param mixed       &$message  Either the message text (string)
+     *                               or a MIME_Message object that
+     *                               contains the text to send.
+     * @param string       $charset  The charset that was used for the
+     *                               headers.
      *
      * @return mixed  True on success, PEAR_Error object on error.
      */
@@ -136,13 +131,27 @@ class IMP_Compose {
             return $mailer;
         }
 
-        return $mailer->send(MIME::encodeAddress($email, null, $_SESSION['imp']['maildomain']), $headerArray, $msg);
+        /* Properly encode the addresses we're sending to. */
+        $email = MIME::encodeAddress($email, null, $_SESSION['imp']['maildomain']);
+        if (is_a($email, 'PEAR_Error')) {
+            return $email;
+        }
+
+        /* Validate the recipient addresses. */
+        require_once 'Mail/RFC822.php';
+        $parser = &new Mail_RFC822();
+        $result = $parser->parseAddressList($email, $_SESSION['imp']['maildomain'], null, true);
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        }
+
+        return $mailer->send($email, $headerArray, $msg);
     }
 
     /**
      * Finds the main "body" text part (if any) in a message.
      *
-     * @param object IMP_Contents &$imp_contents  An IMP_Contents object.
+     * @param IMP_Contents &$imp_contents  An IMP_Contents object.
      *
      * @return string  The text of the "body" part of the message.
      *                 Returns an empty string if no "body" found.
@@ -167,7 +176,7 @@ class IMP_Compose {
 
         if ($mime_part->getSubType() == 'html') {
             require_once 'Horde/Text/Filter.php';
-            return Text_Filter::filter($body, 'html2text');
+            return Text_Filter::filter($body, 'html2text', array('wrap' => ($mime_part->getContentTypeParameter('format') != 'flowed'), 'charset' => $mime_part->getCharset()));
         } else {
             return $body;
         }
@@ -176,7 +185,7 @@ class IMP_Compose {
     /**
      * Returns the ID of the MIME part containing the "body".
      *
-     * @param object IMP_Contents &$imp_contents  An IMP_Contents object.
+     * @param IMP_Contents &$imp_contents  An IMP_Contents object.
      *
      * @return string  The ID of the mime part's body.
      */
@@ -191,11 +200,11 @@ class IMP_Compose {
     /**
      * Determine the reply text for a message.
      *
-     * @param object IMP_Contents &$imp_contents  An IMP_Contents object.
-     * @param string $from                        The email address of the
-     *                                            original sender.
-     * @param object IMP_Headers &$h              The IMP_Headers object for
-     *                                            the message.
+     * @param IMP_Contents &$imp_contents  An IMP_Contents object.
+     * @param string        $from          The email address of the
+     *                                     original sender.
+     * @param IMP_Headers  &$h             The IMP_Headers object for
+     *                                     the message.
      *
      * @return string  The text of the body part of the message to use
      *                 for the reply.
@@ -271,9 +280,9 @@ class IMP_Compose {
     /**
      * Determine the text for a forwarded message.
      *
-     * @param object IMP_Contents &$imp_contents  An IMP_Contents object.
-     * @param object IMP_Headers &$h              The IMP_Headers object for
-     *                                            the message.
+     * @param IMP_Contents &$imp_contents  An IMP_Contents object.
+     * @param IMP_Headers &$h              The IMP_Headers object for
+     *                                     the message.
      *
      * @return string  The text of the body part of the message to use
      *                 for the forward.
@@ -329,8 +338,6 @@ class IMP_Compose {
      * MIME_Part information entries 'temp_filename' and 'temp_filetype'
      * are set with this information.
      *
-     * @access public
-     *
      * @param string $name         The input field name from the form.
      * @param string $disposition  The disposition to use for the file.
      *
@@ -346,7 +353,7 @@ class IMP_Compose {
             return $res;
         }
 
-        $filename = basename($_FILES[$name]['name']);
+        $filename = $_FILES[$name]['name'];
         $tempfile = $_FILES[$name]['tmp_name'];
 
         /* Check for filesize limitations. */
@@ -364,7 +371,7 @@ class IMP_Compose {
             require_once 'Horde/MIME/Magic.php';
             /* Try to determine the MIME type from 1) analysis of the file
                (if available) and, if that fails, 2) from the extension. */
-            if (!($type = MIME_Magic::analyzeFile($tempfile))) {
+            if (!($type = MIME_Magic::analyzeFile($tempfile, isset($conf['mime']['magic_db']) ? $conf['mime']['magic_db'] : null))) {
                 $type = MIME_Magic::filenameToMIME($filename, false);
             }
             $part = &new MIME_Part($type);
@@ -389,18 +396,16 @@ class IMP_Compose {
             return $result;
         }
 
-        return $part->getName();
+        return $filename;
     }
 
     /**
      * Adds an attachment to a MIME_Part from data existing in the part.
      *
-     * @access public
+     * @param MIME_Part &$part  The MIME_Part object that contains the
+     *                          attachment data.
      *
-     * @param object MIME_Part &$part  The MIME_Part object that contains the
-     *                                 attachment data.
-     *
-     * @return object PEAR_Error  Returns a PEAR_Error object on error.
+     * @return PEAR_Error  Returns a PEAR_Error object on error.
      */
     function addMIMEPartAttachment(&$part)
     {
@@ -458,11 +463,10 @@ class IMP_Compose {
      *
      * @access private
      *
-     * @param object MIME_Part &$part     The MIME_Part of the attachment.
-     * @param string $data                Either the filename of the
-     *                                    attachment or, if $vfs_file is
-     *                                    false, the attachment data.
-     * @param optional boolean $vfs_file  If using VFS, is $data a filename?
+     * @param MIME_Part &$part   The MIME_Part of the attachment.
+     * @param string $data       Either the filename of the attachment or, if
+     *                           $vfs_file is false, the attachment data.
+     * @param boolean $vfs_file  If using VFS, is $data a filename?
      */
     function _storeAttachment(&$part, $data, $vfs_file = true)
     {
@@ -500,8 +504,6 @@ class IMP_Compose {
     /**
      * Delete attached files.
      *
-     * @access public
-     *
      * @param mixed $number  Either a single integer or an array of integers
      *                       corresponding to the attachment position.
      *
@@ -550,8 +552,6 @@ class IMP_Compose {
 
     /**
      * Deletes all attachments.
-     *
-     * @access public
      */
     function deleteAllAttachments()
     {
@@ -567,8 +567,6 @@ class IMP_Compose {
     /**
      * Updates information in a specific attachment.
      *
-     * @access public
-     *
      * @param integer $number  The attachment to update.
      * @param array $params    An array of update information.
      * <pre>
@@ -586,8 +584,6 @@ class IMP_Compose {
     /**
      * Returns the list of current attachments.
      *
-     * @access public
-     *
      * @return array  The list of attachments.
      */
     function getAttachments()
@@ -598,8 +594,6 @@ class IMP_Compose {
     /**
      * Returns the number of attachments currently in this message.
      *
-     * @access public
-     *
      * @return integer  The number of attachments in this message.
      */
     function numberOfAttachments()
@@ -610,8 +604,6 @@ class IMP_Compose {
     /**
      * Returns the size of the attachments in bytes.
      *
-     * @access public
-     *
      * @return integer  The size of the attachments (in bytes).
      */
     function sizeOfAttachments()
@@ -622,11 +614,9 @@ class IMP_Compose {
     /**
      * Build a single attachment part with its data.
      *
-     * @access public
-     *
      * @param integer $id  The ID of the part to rebuild.
      *
-     * @return object MIME_Part  The MIME_Part with its contents.
+     * @return MIME_Part  The MIME_Part with its contents.
      */
     function buildAttachment($id)
     {
@@ -638,17 +628,22 @@ class IMP_Compose {
     /**
      * Build the MIME_Part attachments from the temporary file data.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$base  The base MIME_Part object to add the
-     *                                 attachments to.
+     * @param MIME_Part &$base  The base MIME_Part object to add the
+     *                          attachments to.
+     * @param string $charset   The charset to use for the filename.
      */
-    function buildAllAttachments(&$base)
+    function buildAllAttachments(&$base, $charset = null)
     {
         foreach ($this->_cache as $part) {
             /* Store the data inside the current part. */
             $this->_buildPartData($part);
 
+            /* Convert the charset of the filename, if necessary. */
+            if (!is_null($charset)) {
+                $part->setName(String::convertCharset($part->getName(true), $part->getCharset(), $charset));
+                $part->setCharset($charset);
+            }
+
             /* Add to the base part. */
             $base->addPart($part);
         }
@@ -660,7 +655,7 @@ class IMP_Compose {
      *
      * @access private
      *
-     * @param object MIME_Part &$part  The part to rebuild data into.
+     * @param MIME_Part &$part  The part to rebuild data into.
      */
     function _buildPartData(&$part)
     {
@@ -686,10 +681,10 @@ class IMP_Compose {
      *
      * @access private
      *
-     * @param string $line            The line of attribution text.
-     * @param string $from            The email address of the original
-     *                                sender.
-     * @param object IMP_Headers &$h  The IMP_Headers object for the message.
+     * @param string $line     The line of attribution text.
+     * @param string $from     The email address of the original
+     *                         sender.
+     * @param IMP_Headers &$h  The IMP_Headers object for the message.
      *
      * @return string  The attribution text.
      */
@@ -713,7 +708,7 @@ class IMP_Compose {
                 }
                 $addressList .= $entry->mailbox;
             }
-            if (isset($entry->personal)){
+            if (isset($entry->personal)) {
                 if (strlen($nameList) > 0) {
                     $nameList .= ', ';
                 }
@@ -727,6 +722,9 @@ class IMP_Compose {
         }
 
         /* Define the macros. */
+        if (is_array($message_id = $h->getOb('message_id'))) {
+            $message_id = reset($message_id);
+        }
         if (!($subject = $h->getOb('subject', true))) {
             $subject = _("[No Subject]");
         }
@@ -761,7 +759,7 @@ class IMP_Compose {
             '/%c/' => String::convertCharset(@strftime("%c", $udate), NLS::getExternalCharset()),
 
             /* Message-ID. */
-            '/%m/' => $h->getOb('message_id'),
+            '/%m/' => $message_id,
 
             /* Message subject. */
             '/%s/' => $subject
@@ -795,8 +793,6 @@ class IMP_Compose {
      * This function needs to be run at least once per pageload to save the
      * session object.
      *
-     * @access public
-     *
      * @return string  The message cache ID if the object needs to be saved.
      *                 Else, false is returned.
      */
@@ -818,8 +814,6 @@ class IMP_Compose {
     /**
      * How many more attachments are allowed?
      *
-     * @access public
-     *
      * @return mixed  Returns true if no attachment limit.
      *                Else returns the number of additional attachments
      *                allowed.
@@ -838,8 +832,6 @@ class IMP_Compose {
     /**
      * What is the maximum attachment size allowed?
      *
-     * @access public
-     *
      * @return integer  The maximum attachment size allowed (in bytes).
      */
     function maxAttachmentSize()
@@ -858,22 +850,28 @@ class IMP_Compose {
     /**
      * Adds the attachments to the message (in the case of a forward with
      * attachments).
+     * This function MUST be called after IMP_Compose::forwardMessage().
      *
-     * @param object MIME_Message &$message  The MIME_Message object.
+     * @param IMP_Contents &$contents  An IMP_Contents object.
      *
      * @return array  An array of PEAR_Error object on error.
      *                An empty array if successful.
      */
-    function attachFilesFromMessage(&$message)
+    function attachFilesFromMessage(&$contents)
     {
         $errors = array();
 
-        foreach ($message->getParts() as $ref => $mime) {
-            if (($ref != 1) ||
-                ($mime->getPrimaryType() != MIME::type(TYPETEXT))) {
-                $res = $this->addMIMEPartAttachment($mime);
-                if (is_a($res, 'PEAR_Error')) {
-                    $errors[] = $res;
+        $dl_list = $contents->getDownloadAllList(true);
+        $mime_message = $contents->getMIMEMessage();
+
+        foreach ($dl_list as $val) {
+            if (is_null($this->_mimeid) || ($val != $this->_mimeid)) {
+                $mime = $mime_message->getPart($val);
+                if (!empty($mime)) {
+                    $res = $this->addMIMEPartAttachment($mime);
+                    if (is_a($res, 'PEAR_Error')) {
+                        $errors[] = $res;
+                    }
                 }
             }
         }
@@ -885,11 +883,9 @@ class IMP_Compose {
      * Convert a text/html MIME_Part message with embedded image links to
      * a multipart/related MIME_Part with the image data embedded in the part.
      *
-     * @access public
+     * @param MIME_Part $mime_part  The text/html MIME_Part object.
      *
-     * @param object MIME_Part $mime_part  The text/html MIME_Part object.
-     *
-     * @return object MIME_Part  The modified MIME_Part.
+     * @return MIME_Part  The modified MIME_Part.
      */
     function convertToMultipartRelated($mime_part)
     {
@@ -964,17 +960,15 @@ class IMP_Compose {
      * attachments to a new folder and remove the MIME_Parts for the
      * attachments.
      *
-     * @access public
+     * @param string    $baseurl    The base URL for creating the links.
+     * @param MIME_Part $base_part  The body of the message.
+     * @param string    $auth       The authorized user who owns the attachments.
      *
-     * @param string           $baseurl    The base URL for creating the links.
-     * @param object MIME_Part $base_part  The body of the message.
-     * @param string           $auth       The authorized user who owns the attachments.
-     *
-     * @returns object MIME_Part  Modified part with links to attachments.
+     * @return MIME_Part  Modified part with links to attachments.
      */
     function linkAttachments($baseurl, $base_part, $auth)
     {
-        global $conf;
+        global $conf, $prefs;
 
         if (!$conf['compose']['link_attachments']) {
             return PEAR::raiseError(_("Linked attachments are forbidden."));
@@ -988,6 +982,15 @@ class IMP_Compose {
         $vfs->autoCreatePath($fullpath);
 
         $trailer = "\n--\n" . String::convertCharset(_("Attachments"), NLS::getCharset(), $base_part->getCharset());
+
+        if ($prefs->getValue('delete_attachments_monthly')) {
+            /* Determine the first day of the month in which the current
+             * attachments will be ripe for deletion, then subtract 1 second
+             * to obtain the last day of the previous month. */
+            $del_time = gmmktime(0, 0, 0, date('n') + $prefs->getValue('delete_attachments_monthly_keep') + 1, 1, date('Y')) - 1;
+            $trailer .= String::convertCharset(' (' . sprintf(_("Links will expire on %s"), date('F j, Y', $del_time)) . ')', NLS::getCharset(), $base_part->getCharset());
+        }
+
         foreach ($this->_cache as $att) {
             $trailer .= "\n" . Util::addParameter($baseurl, array('u' => $auth,
                                                                   't' => $ts,
@@ -1002,6 +1005,7 @@ class IMP_Compose {
                 $vfs->writeData($fullpath, escapeshellcmd($att->getName()), $data);
             }
         }
+
         $this->deleteAllAttachments();
         $base_part->appendContents($trailer, $base_part->getCurrentEncoding());
 
