@@ -2,22 +2,23 @@
 
 require_once 'Horde/Crypt/pgp.php';
 
-/** @constant string IMP_PGP_PUBKEY_FIELD  Name of PGP public key field in addressbook. */
+/**
+ * Name of PGP public key field in addressbook.
+ */
 define('IMP_PGP_PUBKEY_FIELD', 'pgpPublicKey');
 
 /**
  * The IMP_PGP:: class contains all functions related to handling
  * PGP messages within IMP.
  *
- * $Horde: imp/lib/Crypt/PGP.php,v 1.90 2004/12/10 05:52:42 slusarz Exp $
+ * $Horde: imp/lib/Crypt/PGP.php,v 1.90.2.9 2005/07/03 03:42:43 selsky Exp $
  *
- * Copyright 2002-2004 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.90 $
  * @since   IMP 4.0
  * @package IMP
  */
@@ -26,14 +27,12 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * The list of available sources to search for keys.
      *
-     * @var array $_sources
+     * @var array
      */
     var $_sources = array();
 
     /**
      * Constructor
-     *
-     * @access public
      */
     function IMP_PGP()
     {
@@ -52,15 +51,13 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Generate the personal Public/Private keypair and store in prefs.
      *
-     * @access public
-     *
      * @param string $realname            See Horde_Crypt_pgp::
      * @param string $email               See Horde_Crypt_pgp::
      * @param string $passphrase          See Horde_Crypt_pgp::
-     * @param optional string $comment    See Horde_Crypt_pgp::
-     * @param optional string $keylength  See Horde_Crypt_pgp::
+     * @param string $comment             See Horde_Crypt_pgp::
+     * @param string $keylength           See Horde_Crypt_pgp::
      *
-     * @return object PEAR_Error  Returns a PEAR_Error object on error.
+     * @return PEAR_Error  Returns a PEAR_Error object on error.
      */
     function generatePersonalKeys($name, $email, $passphrase, $comment = '',
                                   $keylength = 1024)
@@ -76,8 +73,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Add the personal public key to the prefs.
      *
-     * @access public
-     *
      * @param mixed $public_key  The public key to add (either string or
      *                           array).
      */
@@ -89,8 +84,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Add the personal private key to the prefs.
      *
-     * @access public
-     *
      * @param mixed $private_key  The private key to add (either string or
      *                            array).
      */
@@ -102,8 +95,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Get the personal public key from the prefs.
      *
-     * @access public
-     *
      * @return string  The personal PGP public key.
      */
     function getPersonalPublicKey()
@@ -114,8 +105,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Get the personal private key from the prefs.
      *
-     * @access public
-     *
      * @return string  The personal PGP private key.
      */
     function getPersonalPrivateKey()
@@ -125,8 +114,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
 
     /**
      * Deletes the specified personal keys from the prefs.
-     *
-     * @access public
      */
     function deletePersonalKeys()
     {
@@ -139,8 +126,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Add a public key to an address book.
      *
-     * @access public
-     *
      * @param string $public_key  An PGP public key.
      *
      * @return array  See Horde_Crypt_pgp::pgpPacketInformation()
@@ -185,13 +170,11 @@ class IMP_PGP extends Horde_Crypt_pgp {
      * Second, if unsuccessful, the key is attempted to be retrieved via
      * a public PGP keyserver.
      *
-     * @access public
+     * @param string $address      The e-mail address to search by.
+     * @param string $fingerprint  The fingerprint of the user's key.
      *
-     * @param string $address               The e-mail address to search by.
-     * @param optional string $fingerprint  The fingerprint of the user's key.
-     *
-     * @return string  The PGP public key requested.
-     *                 Returns PEAR_Error object on error.
+     * @return string  The PGP public key requested. Returns PEAR_Error object
+     *                 on error.
      */
     function getPublicKey($address, $fingerprint = null)
     {
@@ -221,10 +204,8 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Retrieves all public keys from a user's address book(s).
      *
-     * @access public
-     *
-     * @return array  All PGP public keys available.
-     *                Returns PEAR_Error object on error.
+     * @return array  All PGP public keys available. Returns PEAR_Error object
+     *                on error.
      *
      */
     function listPublicKeys()
@@ -239,11 +220,9 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Deletes a public key from a user's address book(s) by e-mail.
      *
-     * @access public
-     *
      * @param string $email  The e-mail address to delete.
      *
-     * @return object PEAR_Error  Returns PEAR_Error object on error.
+     * @return PEAR_Error  Returns PEAR_Error object on error.
      */
     function deletePublicKey($email)
     {
@@ -253,8 +232,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Parse a message into its PGP components.
      *
-     * @access public
-     *
      * @param string $text  See Horde_Crypt_pgp::parsePGPData()
      *
      * @return array  Returns an array of MIME_Part objects.
@@ -308,12 +285,10 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Renders a text message with PGP components.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$part          The MIME_Part containing the
-     *                                         data to render.
-     * @param object MIME_Contents &$contents  The MIME_Contents:: module to
-     *                                         use to output the text.
+     * @param MIME_Part &$part          The MIME_Part containing the data to
+     *                                  render.
+     * @param MIME_Contents &$contents  The MIME_Contents:: module to use to
+     *                                  output the text.
      *
      * @return string  Returns the rendered text.
      *                 If there was no PGP data, returns false.
@@ -355,11 +330,8 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Returns the signed data only for a plaintext signed MIME_Part.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object with a
-     *                                      plaintext PGP signed message in the
-     *                                      contents.
+     * @param MIME_Part &$mime_part  The MIME_Part object with a plaintext PGP
+     *                               signed message in the contents.
      *
      * @return string  The contents of the signed message.
      */
@@ -404,8 +376,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Get a public key via a public PGP keyserver.
      *
-     * @access public
-     *
      * @param string $fingerprint  The fingerprint of the requested key.
      *
      * @return string  See Horde_Crypt_pgp::getPublicKeyserver()
@@ -418,8 +388,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Send a public key to a public PGP keyserver.
      *
-     * @access public
-     *
      * @param string $pubkey  The PGP public key.
      *
      * @return string  See Horde_Crypt_pgp::putPublicKeyserver()
@@ -465,11 +433,9 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Verifies a signed message with a given public key.
      *
-     * @access public
-     *
-     * @param string $text                The text to verify.
-     * @param string $address             E-mail address of public key.
-     * @param optional string $signature  A PGP signature block.
+     * @param string $text       The text to verify.
+     * @param string $address    E-mail address of public key.
+     * @param string $signature  A PGP signature block.
      *
      * @return string  See Horde_Crypt_pgp::decryptSignature()
      *                 -OR-
@@ -509,12 +475,10 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Decrypt a message with user's public/private keypair.
      *
-     * @access public
-     *
      * @param string $text  The text to decrypt.
      *
-     * @return string  The decrypted message.
-     *                 Returns PEAR_Error object on error.
+     * @return string  The decrypted message. Returns PEAR_Error object on
+     *                 error.
      */
     function decryptMessage($text)
     {
@@ -525,8 +489,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Gets the user's passphrase from the session cache.
      *
-     * @access public
-     *
      * @return string  The passphrase, if set.
      */
     function getPassphrase()
@@ -541,8 +503,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Store's the user's passphrase in the session cache.
      *
-     * @access public
-     *
      * @param string $passphrase  The user's passphrase.
      *
      * @return boolean  Returns true if correct passphrase, false if incorrect.
@@ -560,8 +520,6 @@ class IMP_PGP extends Horde_Crypt_pgp {
 
     /**
      * Clear the passphrase from the session cache.
-     *
-     * @access public
      */
     function unsetPassphrase()
     {
@@ -571,11 +529,8 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Generates the javascript code for saving public keys.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part containing the
-     *                                      public key.
-     * @param optional string $cache        The MIME_Part identifier.
+     * @param MIME_Part &$mime_part  The MIME_Part containing the public key.
+     * @param string $cache          The MIME_Part identifier.
      *
      * @return string  The URL for saving public keys.
      */
@@ -593,34 +548,32 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Print out the link for the open_pgp_win javascript function.
      *
-     * @access public
-     *
-     * @param string $actionid        The ActionID to perform.
-     * @param optional mixed $reload  If true, reload base window on close.
-     *                                If text, run this JS on close.
-     *                                If false, don't do anything on close.
-     * @param optional array $params  Additional parameters needed for the
-     *                                reload page.
+     * @param string $actionid  The ActionID to perform.
+     * @param mixed $reload     If true, reload base window on close. If text,
+     *                          run this JS on close. If false, don't do
+     *                          anything on close.
+     * @param array $params     Additional parameters needed for the reload
+     *                          page.
      *
      * @return string  The javascript link.
      */
     function getJSOpenWinCode($actionid, $reload = true, $params = null)
     {
-        $popup_url = Horde::applicationUrl('pgp.php');
-        $popup_url = Util::addParameter($popup_url, 'actionID', $actionid);
+        $popup_url = Horde::applicationUrl('pgp.php', true);
+        $popup_url = Util::addParameter($popup_url, 'actionID', $actionid, false);
         if (!empty($reload)) {
             if (is_bool($reload)) {
-                $popup_url = Util::addParameter($popup_url, 'reload', Horde::selfUrl(true));
+                $popup_url = Util::addParameter($popup_url, 'reload', Horde::selfUrl(true), false);
             } else {
                 require_once 'Horde/SessionObjects.php';
                 $cacheSess = &Horde_SessionObjects::singleton();
-                $popup_url = Util::addParameter($popup_url, 'passphrase_action', $cacheSess->storeOid($reload, false));
+                $popup_url = Util::addParameter($popup_url, 'passphrase_action', $cacheSess->storeOid($reload, false), false);
             }
         }
 
         if (is_array($params)) {
             foreach ($params as $val) {
-                $popup_url = Util::addParameter($popup_url, $val);
+                $popup_url = Util::addParameter($popup_url, $val, null, false);
             }
         }
 
@@ -668,12 +621,10 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Sign a MIME_Part using PGP using IMP default parameters.
      *
-     * @access public
+     * @param MIME_Part $mime_part  The MIME_Part object to sign.
      *
-     * @param object MIME_Part $mime_part  The MIME_Part object to sign.
-     *
-     * @return object MIME_Part  See Horde_Crypt_pgp::signMIMEPart().
-     *                           Returns PEAR_Error object on error.
+     * @return MIME_Part  See Horde_Crypt_pgp::signMIMEPart(). Returns
+     *                    PEAR_Error object on error.
      */
     function IMPsignMIMEPart($mime_part)
     {
@@ -683,12 +634,12 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Encrypt a MIME_Part using PGP using IMP default parameters.
      *
-     * @param object MIME_Part $mime_part  The MIME_Part object to encrypt.
-     * @param string $to_address           The e-mail address of the key to
-     *                                     use for encryption.
+     * @param MIME_Part $mime_part  The MIME_Part object to encrypt.
+     * @param string $to_address    The e-mail address of the key to use for
+     *                              encryption.
      *
-     * @return object MIME_Part  See Horde_Crypt_pgp::encryptMIMEPart().
-     *                           Returns PEAR_Error object on error.
+     * @return MIME_Part  See Horde_Crypt_pgp::encryptMIMEPart(). Returns
+     *                    PEAR_Error object on error.
      */
     function IMPencryptMIMEPart($mime_part, $to_address)
     {
@@ -702,13 +653,12 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Sign and Encrypt a MIME_Part using PGP using IMP default parameters.
      *
-     * @param object MIME_Part $mime_part  The MIME_Part object to sign and
-     *                                     encrypt.
-     * @param string $to_address           The e-mail address of the key to
-     *                                     use for encryption.
+     * @param MIME_Part $mime_part  The MIME_Part object to sign and encrypt.
+     * @param string $to_address    The e-mail address of the key to use for
+     *                              encryption.
      *
-     * @return object MIME_Part  See Horde_Crypt_pgp::signAndencryptMIMEPart().
-     *                           Returns PEAR_Error object on error.
+     * @return MIME_Part  See Horde_Crypt_pgp::signAndencryptMIMEPart().
+     *                    Returns PEAR_Error object on error.
      */
     function IMPsignAndEncryptMIMEPart($mime_part, $to_address)
     {
@@ -723,7 +673,7 @@ class IMP_PGP extends Horde_Crypt_pgp {
      * Generate a MIME_Part object, in accordance with RFC 2015/3156, that
      * contains the user's public key.
      *
-     * @return object MIME_Part  See Horde_Crypt_pgp::publicKeyMIMEPart().
+     * @return MIME_Part  See Horde_Crypt_pgp::publicKeyMIMEPart().
      */
     function publicKeyMIMEPart()
     {
