@@ -123,11 +123,11 @@ define('IMP_IDX_SEP', "\1");
 /**
  * IMP Base Class.
  *
- * $Horde: imp/lib/IMP.php,v 1.449 2004/12/02 16:27:53 chuck Exp $
+ * $Horde: imp/lib/IMP.php,v 1.449.4.28 2005/09/01 10:29:22 jan Exp $
  *
- * Copyright 1999-2004 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2004 Jon Parise <jon@horde.org>
- * Copyright 2002-2004 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you did not
  * receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -135,7 +135,6 @@ define('IMP_IDX_SEP', "\1");
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@horde.org>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.449 $
  * @since   IMP 2.3.5
  * @package IMP
  */
@@ -144,9 +143,7 @@ class IMP {
     /**
      * Returns the AutoLogin server key.
      *
-     * @access public
-     *
-     * @param optional boolean $first  Return the first value?
+     * @param boolean $first  Return the first value?
      *
      * @return string  The server key.
      */
@@ -195,8 +192,6 @@ class IMP {
     /**
      * Makes sure the user has been authenticated to view the page.
      *
-     * @access public
-     *
      * @param mixed $flags     Any flags to pass to imap_open(). See
      *                         Auth_imp::authenticate(). However, if this is
      *                         the string 'horde', we just check for Horde auth
@@ -321,8 +316,6 @@ class IMP {
     /**
      * Generates a full c-client server specification string.
      *
-     * @access public
-     *
      * @param string $mbox      The mailbox to append to end of the server
      *                          string.
      * @param string $protocol  Override the protocol currently being used.
@@ -355,28 +348,24 @@ class IMP {
      * replacing IMP_SEARCH_MBOX with an appropriate string and removing
      * namespace and folder prefix information from what is shown to the user.
      *
-     * @access public
-     *
      * @return string  The plain text label.
      */
     function getLabel()
     {
-        global $imp;
-
         $label = '';
 
-        if ($imp['mailbox'] == 'INBOX') {
+        if ($_SESSION['imp']['mailbox'] == 'INBOX') {
             $label = _("Inbox");
-        } elseif ($imp['mailbox'] == IMP_SEARCH_MBOX) {
+        } elseif ($_SESSION['imp']['mailbox'] == IMP_SEARCH_MBOX) {
             require_once IMP_BASE . '/lib/Search.php';
             $label = IMP_Search::searchLabel();
         } else {
-            $label = String::convertCharset($imp['mailbox'], 'UTF7-IMAP');
-            if (strcmp($imp['folders'], String::substr($label, 0, String::length($imp['folders']))) == 0) {
-                $label = String::substr($label, String::length($imp['folders']));
+            $label = String::convertCharset($_SESSION['imp']['mailbox'], 'UTF7-IMAP');
+            if (strcmp($_SESSION['imp']['folders'], String::substr($label, 0, String::length($_SESSION['imp']['folders']))) == 0) {
+                $label = String::substr($label, String::length($_SESSION['imp']['folders']));
             }
-            if (strcmp($imp['namespace'], String::substr($label, 0, String::length($imp['namespace']))) == 0) {
-                $label = String::substr($label, String::length($imp['namespace']));
+            if (strcmp($_SESSION['imp']['namespace'], String::substr($label, 0, String::length($_SESSION['imp']['namespace']))) == 0) {
+                $label = String::substr($label, String::length($_SESSION['imp']['namespace']));
             }
         }
 
@@ -386,8 +375,6 @@ class IMP {
     /**
      * Returns the bare address.
      *
-     * @access public
-     *
      * @param string $address    The address string.
      * @param boolean $multiple  Should we return multiple results?
      *
@@ -395,19 +382,23 @@ class IMP {
      */
     function bareAddress($address, $multiple = false)
     {
-        global $imp;
+        static $addresses;
 
-        require_once 'Horde/MIME.php';
-        return MIME::bareAddress($address, $imp['maildomain'], $multiple);
+        if (!isset($addresses[(string)$multiple][$address])) {
+            require_once 'Horde/MIME.php';
+            $addresses[(string)$multiple][$address] = MIME::bareAddress($address, $_SESSION['imp']['maildomain'], $multiple);
+        }
+
+        return $addresses[(string)$multiple][$address];
     }
 
     /**
      * Uses the Registry to expand names and returning error information for
      * any address that is either not valid or fails to expand.
      *
-     * @param string $addrString      The name(s) or address(es) to expand.
-     * @param optional boolean $full  If true generate a full, rfc822-valid
-     *                                address list.
+     * @param string $addrString  The name(s) or address(es) to expand.
+     * @param boolean $full       If true generate a full, rfc822-valid address
+     *                            list.
      *
      * @return mixed   Either a string containing all expanded addresses or an
      *                 array containing all matching address or an error
@@ -448,6 +439,7 @@ class IMP {
 
         $arr = MIME::rfc822Explode($addrString, ',');
         $arr = array_map('trim', $arr);
+        $arr = array_filter($arr);
 
         $results = $GLOBALS['registry']->call('contacts/search', array($arr, $src, $search_fields));
         if (is_a($results, 'PEAR_Error')) {
@@ -456,53 +448,56 @@ class IMP {
 
         $ambiguous = false;
         $error = false;
-        $i = 0;
         $missing = array();
 
-        foreach ($results as $res) {
-            $tmp = $arr[$i];
-            if ($parser->validateMailbox(MIME::encodeAddress($tmp, null, ''))) {
+        foreach ($arr as $i => $tmp) {
+            $address = MIME::encodeAddress($tmp, null, '');
+            if (!is_a($address, 'PEAR_Error') &&
+                ($parser->validateMailbox($address))) {
                 // noop
-            } elseif (count($res) == 1) {
-                if ($full) {
-                    if (strpos($res[0]['email'], ',') !== false) {
-                        $arr[$i] = $res[0]['name'] . ': ' . $res[0]['email'] . ';';
-                    } else {
-                        list($mbox, $host) = explode('@', $res[0]['email']);
-                        $arr[$i] = MIME::rfc822WriteAddress($mbox, $host, $res[0]['name']);
-                    }
-                } else {
-                    $arr[$i] = $res[0]['name'];
-                }
-            } elseif (count($res) > 1) {
-                /* Handle the multiple case - we return an array with all found
-                   addresses. */
-                $arr[$i] = array($arr[$i]);
-                foreach ($res as $one_res) {
-                    if ($full) {
-                        if (strpos($one_res['email'], ',') !== false) {
-                            $arr[$i][] = $one_res['name'] . ': ' . $one_res['email'] . ';';
-                        } else {
-                            $mbox_host = explode('@', $one_res['email']);
-                            if (isset($mbox_host[1])) {
-                                $arr[$i][] = MIME::rfc822WriteAddress($mbox_host[0], $mbox_host[1], $one_res['name']);
-                            }
-                        }
-                    } else {
-                        $arr[$i][] = $one_res['name'];
-                    }
-                }
-                $ambiguous = true;
-            } else {
-                /* Handle the missing/invalid case - we should return error info
-                   on each address that couldn't be expanded/validated. */
+            } elseif (!isset($results[$tmp]) || !count($results[$tmp])) {
+                /* Handle the missing/invalid case - we should return error
+                 * info on each address that couldn't be
+                 * expanded/validated. */
                 $error = true;
                 if (!$ambiguous) {
                     $arr[$i] = PEAR::raiseError(null, null, null, null, $arr[$i]);
                     $missing[$i] = $arr[$i];
                 }
+            } else {
+                $res = $results[$tmp];
+                if (count($res) == 1) {
+                    if ($full) {
+                        if (strpos($res[0]['email'], ',') !== false) {
+                            $arr[$i] = $res[0]['name'] . ': ' . $res[0]['email'] . ';';
+                        } else {
+                            list($mbox, $host) = explode('@', $res[0]['email']);
+                            $arr[$i] = MIME::rfc822WriteAddress($mbox, $host, $res[0]['name']);
+                        }
+                    } else {
+                        $arr[$i] = $res[0]['name'];
+                    }
+                } else {
+                    /* Handle the multiple case - we return an array
+                     * with all found addresses. */
+                    $arr[$i] = array($arr[$i]);
+                    foreach ($res as $one_res) {
+                        if ($full) {
+                            if (strpos($one_res['email'], ',') !== false) {
+                                $arr[$i][] = $one_res['name'] . ': ' . $one_res['email'] . ';';
+                            } else {
+                                $mbox_host = explode('@', $one_res['email']);
+                                if (isset($mbox_host[1])) {
+                                    $arr[$i][] = MIME::rfc822WriteAddress($mbox_host[0], $mbox_host[1], $one_res['name']);
+                                }
+                            }
+                        } else {
+                            $arr[$i][] = $one_res['name'];
+                        }
+                    }
+                    $ambiguous = true;
+                }
             }
-            $i++;
         }
 
         if ($ambiguous) {
@@ -513,7 +508,54 @@ class IMP {
         } elseif ($error) {
             return PEAR::raiseError(_("Please resolve ambiguous or invalid addresses."), null, null, null, $arr);
         } else {
-            return implode(', ', $arr);
+            $list = '';
+            foreach ($arr as $elm) {
+                if (substr($list, -1) == ';') {
+                    $list .= ' ';
+                } elseif (!empty($list)) {
+                    $list .= ', ';
+                }
+                $list .= $elm;
+            }
+            return $list;
+        }
+    }
+
+    /**
+     * Adds a contact to the user defined address book.
+     *
+     * @param string $newAddress  The contact's email address.
+     * @param string $newName     The contact's name.
+     *
+     * @return string  A link or message to show in the notification area.
+     */
+    function addAddress($newAddress, $newName)
+    {
+        global $registry, $prefs;
+
+        if (empty($newName)) {
+            $newName = $newAddress;
+        }
+
+        $result = $registry->call('contacts/import',
+                                  array(array('name' => $newName, 'email' => $newAddress),
+                                        'array', $prefs->getValue('add_source')));
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        } else {
+            if (!preg_match('/[^a-z0-9]/', $result)) {
+                /* Broken contacts/import implementation that returns the
+                 * source key intead of the UID. */
+                $contact_link = @htmlspecialchars($newName, ENT_COMPAT, NLS::getCharset());
+            } else {
+                $contact_link = $registry->link('contacts/show', array('uid' => $result, 'source' => $prefs->getValue('add_source')));
+                if (!empty($contact_link) && !is_a($contact_link, 'PEAR_Error')) {
+                    $contact_link = Horde::link(Horde::url($contact_link), sprintf(_("Go to address book entry of \"%s\""), $newName)) . @htmlspecialchars($newName, ENT_COMPAT, NLS::getCharset()) . '</a>';
+                } else {
+                    $contact_link = @htmlspecialchars($newName, ENT_COMPAT, NLS::getCharset());
+                }
+            }
+            return $contact_link;
         }
     }
 
@@ -523,23 +565,27 @@ class IMP {
      * &lt;select&gt; and &lt;/select&gt; tags are NOT included in the output
      * of this function.
      *
-     * @access public
+     * @param string $heading         The label for an empty-value option at
+     *                                the top of the list.
+     * @param boolean $abbrev         If true, abbreviate long mailbox names
+     *                                by replacing the middle of the name with
+     *                                '...'.
+     * @param array $filter           An array of mailboxes to ignore. If the
+     *                                first element in the array is null, then
+     *                                the mailbox will be shown in the
+     *                                resulting list, but there will be an
+     *                                empty value argument (i.e.
+     *                                non-selectable).
+     * @param string $selected        The mailbox to have selected by default.
+     * @param boolean $new_folder     If true, display an option to create a
+     *                                new folder.
+     * @param boolean $inc_tasklists  Should the user's editable tasklists be
+     *                                included in the list?
+     * @param boolean $inc_vfolder    Should the user's virtual folders be
+     *                                included in the list?
      *
-     * @param string $heading      An optional string to use as the label for
-     *                             an empty-value option at the top of the list
-     * @param bool $abbrev         If true, abbreviate long mailbox names by
-     *                             replacing the middle of the name with '...'.
-     * @param array $filter        An array of mailboxes to ignore.
-     * @param string $selected     The mailbox to have selected by default.
-     * @param bool $new_folder     If true, display an option to create a new
-     *                             folder.
-     * @param bool $inc_tasklists  Should the user's editable tasklists be
-     *                             included in the list?
-     * @param bool $inc_vfolder    Should the user's virtual folders be
-     *                             included in the list?
-     *
-     * @return string  A string containg <option> elements for each mailbox in
-     *                 the list.
+     * @return string  A string containing <option> elements for each mailbox
+     *                 in the list.
      */
     function flistSelect($heading = '', $abbrev = true, $filter = array(),
                          $selected = null, $new_folder = false,
@@ -571,14 +617,20 @@ class IMP {
         }
 
         /* Add the list of mailboxes to the lists. */
+        $showmbox = false;
+        if (!empty($filter) && is_null($filter[0])) {
+            $showmbox = true;
+            array_shift($filter);
+        }
+
         $filter = array_flip($filter);
         foreach ($mailboxes as $mbox) {
-            if (isset($filter[$mbox['val']])) {
+            if (isset($filter[$mbox['val']]) && !$showmbox) {
                 continue;
             }
 
+            $val = isset($filter[$mbox['val']]) ? '' : htmlspecialchars($mbox['val']);
             $sel = ($mbox['val'] && ($mbox['val'] === $selected)) ? ' selected="selected"' : '';
-            $val = htmlspecialchars($mbox['val']);
             $label = ($abbrev) ? $mbox['abbrev'] : $mbox['label'];
             $text .= sprintf('<option value="%s"%s>%s</option>%s', $val, $sel, Text::htmlSpaces($label), "\n");
         }
@@ -589,7 +641,7 @@ class IMP {
             if (!empty($vfolders)) {
                 $text .= '<option value="">----</option>' . "\n";
                 foreach ($vfolders as $id => $val) {
-                    $sel = (($GLOBALS['imp']['mailbox'] == IMP_SEARCH_MBOX) && ($id == $_SESSION['imp']['search']['vfolder_id'])) ? ' selected="selected"' : '';
+                    $sel = (($_SESSION['imp']['mailbox'] == IMP_SEARCH_MBOX) && ($id == $_SESSION['imp']['search']['vfolder_id'])) ? ' selected="selected"' : '';
                     $text .= sprintf('<option value="%s"%s>%s</option>%s', IMP_VFOLDER_PREFIX . $id, $sel, Text::htmlSpaces($val), "\n");
                 }
             }
@@ -618,24 +670,20 @@ class IMP {
     /**
      * Returns the current mailbox.
      *
-     * @access public
-     *
      * @param integer $array_index  The array index of the current message.
      *
      * @return string  The current mailbox name.
      */
     function getThisMailbox($array_index = null)
     {
-        global $imp;
-
-        if (!is_null($array_index) && ($imp['mailbox'] == IMP_SEARCH_MBOX)) {
-            $msgs = explode(IMP_MSG_SEP, $imp['msgl']);
+        if (!is_null($array_index) && ($_SESSION['imp']['mailbox'] == IMP_SEARCH_MBOX)) {
+            $msgs = explode(IMP_MSG_SEP, $_SESSION['imp']['msgl']);
             if (isset($msgs[$array_index])) {
                 return substr($msgs[$array_index], strpos($msgs[$array_index], IMP_IDX_SEP) + 1);
             }
         }
 
-        return $imp['thismailbox'];
+        return isset($_SESSION['imp']['thismailbox']) ? $_SESSION['imp']['thismailbox'] : '';
     }
 
     /**
@@ -643,8 +691,6 @@ class IMP {
      * pass back either a URI fragment or an associative array with any of
      * them which are present.
      *
-     * @access public
-     *
      * @param string $format  Either 'uri' or 'array'.
      *
      * @return string  A URI fragment or an associative array with any compose
@@ -683,8 +729,6 @@ class IMP {
     /**
      * Returns the initial URL.
      *
-     * @access public
-     *
      * @param string $actionID  The action ID to perform on the initial page.
      * @param boolean $encode   If true the argument separator gets encoded.
      *
@@ -732,8 +776,6 @@ class IMP {
     /**
      * Returns the appropriate link to call the message composition screen.
      *
-     * @access public
-     *
      * @param mixed $args   List of arguments to pass to compose.php. If this
      *                      is passed in as a string, it will be parsed as a
      *                      toaddress?subject=foo&cc=ccaddress (mailto-style)
@@ -774,13 +816,14 @@ class IMP {
                 $url = Util::addParameter($url, $key, $val);
             }
         }
+        $url = substr($url, 1);
 
         if ($prefs->getValue('compose_popup') &&
             $browser->hasFeature('javascript')) {
             Horde::addScriptFile('open_compose_win.js');
-            return 'javascript:open_compose_win(\'' . $browser->escapeJSCode(substr($url, 1)) . '\');';
+            return 'javascript:open_compose_win(\'' . addslashes($url) . '\');';
         } else {
-            return Horde::applicationUrl('compose.php' . $url);
+            return Horde::applicationUrl('compose.php?' . $url);
         }
     }
 
@@ -836,15 +879,19 @@ class IMP {
         if (substr($folder, 0, ($prefixLen = strlen($prefix))) == $prefix) {
             $folder = substr($folder, $prefixLen);
         }
+        if ($decode) {
+            $folder = String::convertCharset($folder, 'UTF7-IMAP');
+        }
+        if ($folder == 'INBOX') {
+            $folder = _("INBOX");
+        }
 
-        return $decode ? String::convertCharset($folder, 'UTF7-IMAP') : $folder;
+        return $folder;
     }
 
     /**
      * Filters a string, if requested.
      *
-     * @access public
-     *
      * @param string $text  The text to filter.
      *
      * @return string  The filtered text (if requested).
@@ -863,12 +910,10 @@ class IMP {
 
     /**
      * Build IMP's list of menu items.
-     *
-     * @access public
      */
     function getMenu($returnType = 'object')
     {
-        global $imp, $conf, $prefs, $registry;
+        global $conf, $prefs, $registry;
 
         require_once 'Horde/Menu.php';
 
@@ -879,10 +924,10 @@ class IMP {
         $menu = &new Menu(HORDE_MENU_MASK_ALL & ~HORDE_MENU_MASK_LOGIN);
 
         $menu->add(Util::addParameter($menu_mailbox_url, 'mailbox', 'INBOX'), _("_Inbox"), 'folders/inbox.png');
-        if (($imp['base_protocol'] != 'pop3') && $prefs->getValue('use_trash') && ($prefs->getValue('trash_folder') !== null) && $prefs->getValue('empty_trash_menu')) {
+        if (($_SESSION['imp']['base_protocol'] != 'pop3') && $prefs->getValue('use_trash') && ($prefs->getValue('trash_folder') !== null) && $prefs->getValue('empty_trash_menu')) {
             $menu_trash_url = Util::addParameter($menu_mailbox_url, 'thismailbox', IMP::preambleString() . $prefs->getValue('trash_folder'));
             $menu_trash_url = Util::addParameter($menu_trash_url, 'actionID', 'empty_mailbox');
-            $menu_trash_url = Util::addParameter($menu_trash_url, 'return_url', IMP::getInitialUrl());
+            $menu_trash_url = Util::addParameter($menu_trash_url, 'return_url', Util::addParameter($menu_mailbox_url, 'mailbox', $_SESSION['imp']['mailbox']));
             $menu->add($menu_trash_url, _("Empty _Trash"), 'empty_trash.png', null, null, "d = new Date(); if (d.getTime() - menu_trash_confirmed > 100) { menu_trash_cvalue = window.confirm('" . addslashes(_("Are you sure you wish to empty your trash folder?")) . "'); d = new Date(); menu_trash_confirmed = d.getTime(); } return menu_trash_cvalue;", '__noselection');
         }
         $menu->add(IMP::composeLink(), _("_Compose"), 'compose.png');
@@ -892,9 +937,9 @@ class IMP {
         $menu->add($menu_search_url, _("_Search"), 'search.png', $registry->getImageDir('horde'));
         if ($prefs->getValue('fetchmail_menu')) {
             if ($prefs->getValue('fetchmail_popup')) {
-                $menu->add('#', _("Acco_unts"), 'fetchmail.png', null, 'fetchmail', 'window.open(\'' . Horde::applicationUrl('fetchmail.php'). '\', \'fetchmail\', \'toolbar=no,location=no,status=yes,scrollbars=yes,resizable=yes,width=300,height=450,left=100,top=100\'); return false;');
+                $menu->add('#', _("F_etch Mail"), 'fetchmail.png', null, 'fetchmail', 'window.open(\'' . Horde::applicationUrl('fetchmail.php'). '\', \'fetchmail\', \'toolbar=no,location=no,status=yes,scrollbars=yes,resizable=yes,width=300,height=450,left=100,top=100\'); return false;');
             } else {
-                $menu->add(Horde::applicationUrl('fetchmail.php'), _("Acco_unts"), 'fetchmail.png');
+                $menu->add(Horde::applicationUrl('fetchmail.php'), _("F_etch Mail"), 'fetchmail.png');
             }
         }
         if ($prefs->getValue('filter_menuitem')) {
@@ -941,14 +986,12 @@ class IMP {
 
     /**
      * Outputs IMP's status/notification bar.
-     *
-     * @access public
      */
     function status()
     {
-        global $imp, $notification;
+        global $notification;
 
-        if (isset($imp['stream'])) {
+        if (isset($_SESSION['imp']['stream'])) {
             $alerts = imap_alerts();
             if (is_array($alerts)) {
                 foreach ($alerts as $alert) {
@@ -981,7 +1024,7 @@ class IMP {
         if (!empty($_alert)) {
             if ($count == 1) {
                 $mailboxOpenUrl = Horde::applicationUrl('mailbox.php', true);
-                $mailboxOpenUrl = Util::addParameter($mailboxOpenUrl, 'no_newmail_popup' , 1);
+                $mailboxOpenUrl = Util::addParameter($mailboxOpenUrl, 'no_newmail_popup', 1);
                 $mailboxOpenUrl = Util::addParameter($mailboxOpenUrl, 'mailbox', $_mailbox_message);
 
                 return "if (confirm('" . addslashes(_("You have new mail in the following folder:")) . '\n' .
@@ -998,8 +1041,6 @@ class IMP {
     /**
      * Generates the URL to the prefs page.
      *
-     * @access public
-     *
      * @param boolean $full  Generate full URL?
      *
      * @return string  The URL to the IMP prefs page.
@@ -1009,4 +1050,110 @@ class IMP {
         return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/prefs.php?app=imp', $full);
     }
 
+    /**
+     * Are we currently in "print" mode?
+     *
+     * @param boolean $mode  True if in print mode, false if not.
+     *
+     * @return boolean  Returns true if in "print" mode.
+     */
+    function printMode($mode = null)
+    {
+        static $print = false;
+        if (!is_null($mode)) {
+            $print = $mode;
+        }
+        return $print;
+    }
+
+    /**
+     * Get message indices list.
+     *
+     * @param mixed $indices  The following inputs are allowed:
+     * <pre>
+     * 1. An array of messages indices in the following format:
+     *    msg_id IMP_IDX_SEP msg_folder
+     *    msg_id = Message index of the message
+     *    IMP_IDX_SEP = IMP constant used to separate index/folder
+     *    msg_folder = The full folder name containing the message index
+     * 2. An array with the full folder name as keys and an array of message
+     *    indices as the values.
+     * 3. An IMP_Mailbox object, which will use the current index/folder
+     *    as determined by the object. If an IMP_Mailbox object is used, it
+     *    will be updated after the action is performed.
+     * </pre>
+     *
+     * @return mixed  Returns an array with the folder as key and an array
+     *                of message indices as the value (See #2 above).
+     *                Else, returns false.
+     */
+    function parseIndicesList($indices)
+    {
+        $msgList = array();
+
+        if (is_a($indices, 'IMP_Mailbox')) {
+            $msgIdx = $indices->getIndex(true);
+            if (empty($msgIdx)) {
+                return false;
+            }
+            $parts = explode(IMP_IDX_SEP, $msgIdx);
+            $msgList[$parts[1]][] = $parts[0];
+            return $msgList;
+        }
+
+        if (!is_array($indices)) {
+            return false;
+        }
+        if (!count($indices)) {
+            return array();
+        }
+
+        reset($indices);
+        if (is_array(current($indices))) {
+            /* Build the list of indices/mailboxes to delete if input
+               is of format #1. */
+            foreach ($indices as $mbox => $msgs) {
+                foreach ($msgs as $msgIndex) {
+                    if (strpos($msgIndex, IMP_IDX_SEP) === false) {
+                        $msgList[$mbox][] = $msgIndex;
+                    } else {
+                        list($val, $key) = explode(IMP_IDX_SEP, $msgIndex);
+                        $msgList[$key][] = $val;
+                    }
+                }
+            }
+        } else {
+            /* We are dealing with format #2. Make sure we don't have
+               any duplicate keys. */
+            foreach ($indices as $key => $val) {
+                if (isset($msgList[$key])) {
+                    $msgList[$key] = array_unique(array_merge($val, $msgList[$key]));
+                } else {
+                    $msgList[$key] = is_array($val) ? $val : array($val);
+                }
+            }
+        }
+
+        return $msgList;
+    }
+
+    /**
+     * Either sets or checks the value of the logintasks flag.
+     *
+     * @param integer $set  The value of the flag.
+     *
+     * @return integer  The value of the flag.
+     *                  0 = No login tasks pending
+     *                  1 = Login tasks pending
+     *                  2 = Login tasks pending, previous tasks interrupted
+     */
+    function loginTasksFlag($set = null)
+    {
+        if (!is_null($set)) {
+            $_SESSION['imp']['_logintasks'] = $set;
+        }
+
+        return isset($_SESSION['imp']['_logintasks']) ? $_SESSION['imp']['_logintasks'] : 0;
+    }
+
 }
