@@ -3,15 +3,14 @@
  * The IMP_MIME_Viewer_multipart class handles multipart messages not
  * rendered by any specific MIME_Viewer.
  *
- * $Horde: imp/lib/MIME/Viewer/multipart.php,v 1.13 2004/05/20 15:45:33 jan Exp $
+ * $Horde: imp/lib/MIME/Viewer/multipart.php,v 1.13.10.4 2005/07/03 03:42:45 selsky Exp $
  *
- * Copyright 2002-2004 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.13 $
  * @since   IMP 4.0
  * @package Horde_MIME_Viewer
  */
@@ -20,8 +19,6 @@ class IMP_MIME_Viewer_multipart extends MIME_Viewer {
     /**
      * Render out the currently set contents.
      *
-     * @access public
-     *
      * @param array $params  An array with a reference to a MIME_Contents
      *                       object.
      *
@@ -39,13 +36,11 @@ class IMP_MIME_Viewer_multipart extends MIME_Viewer {
     /**
      * Return the content-type.
      *
-     * @access public
-     *
      * @return string  The content-type of the message.
      */
     function getType()
     {
-        return 'text/html';
+        return 'text/html; charset=' . NLS::getCharset();
     }
 
 }
